import { r as registerInstance, c as createEvent, h } from './index-996ea621.js';
import { W as WeekStartDay, D as DateUtils } from './dateutils-5967aeac.js';
import { a as CalendarItem } from './utils-49f2cbe6.js';

const K2btCalendarDayInMonthPicker = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.dayClicked = createEvent(this, "dayClicked", 7);
    this.weekstartday = WeekStartDay.Sunday;
  }
  componentWillRender() {
    this.currentMonth = this.selectedDate.getMonth();
    this.currentYear = this.selectedDate.getFullYear();
  }
  render() {
    const currentDate = new Date();
    const monthName = DateUtils.getMonthName(this.currentMonth);
    const weeks = DateUtils.getWeeksOfMonth(this.currentYear, this.currentMonth + 1, this.weekstartday);
    const firstDayOfMonth = new Date(this.currentYear, this.currentMonth, 1);
    const lastDayOfMonth = new Date(this.currentYear, this.currentMonth + 1, 0);
    return (h("div", { class: "K2BT_CalendarPickerContainer" }, h("div", { class: "K2BT_CalendarPickerHeader" }, h("div", { class: "K2BT_CalendarPickerPreviousMonth", onClick: () => this.previousMonth() }), h("div", { class: "K2BT_CalendarPickerCurrentMonth" }, monthName, " ", this.currentYear), h("div", { class: "K2BT_CalendarPickerNextMonth", onClick: () => this.nextMonth() })), h("div", { class: "K2BT_CalendarPickerWeekdays" }, weeks[0]
      .map(d => d.getDay())
      .map(weekdayNum => (h("div", { class: "K2BT_CalendarPickerWeekday" }, DateUtils.getDayOfWeekShortStringFromDayNumber(weekdayNum))))), h("div", { class: "K2BT_CalendarPickerCalendar" }, weeks.map(week => (h("div", { class: "K2BT_CalendarPickerWeek" }, week.map(date => {
      const dayClass = DateUtils.equalsDate(date, currentDate) ? 'K2BT_CalendarPickerDay K2BT_CalendarPickerToday' : 'K2BT_CalendarPickerDay';
      return date >= firstDayOfMonth && date <= lastDayOfMonth ? (h("div", { class: dayClass, onClick: () => this.dayClickHandler(date) }, date.getDate())) : (h("div", { class: "K2BT_CalendarPickerDayPlaceholder" }));
    })))))));
  }
  previousMonth() {
    if (this.currentMonth == 0) {
      this.currentMonth = 11;
      this.currentYear = this.currentYear - 1;
    }
    else {
      this.currentMonth = this.currentMonth - 1;
    }
    this.updateSelectedDate();
  }
  nextMonth() {
    if (this.currentMonth == 11) {
      this.currentMonth = 0;
      this.currentYear = this.currentYear + 1;
    }
    else {
      this.currentMonth = this.currentMonth + 1;
    }
    this.updateSelectedDate();
  }
  updateSelectedDate() {
    this.selectedDate = new Date(this.currentYear, this.currentMonth, 1, 0, 0, 0);
  }
  dayClickHandler(date) {
    if (!date)
      return; // Ignore clicks on empty days
    this.dayClicked.emit(date);
  }
};

const K2btCalendarMonthView = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.dayClicked = createEvent(this, "dayClicked", 7);
    this.itemClicked = createEvent(this, "itemClicked", 7);
    this.actionClicked = createEvent(this, "actionClicked", 7);
    this.readonly = true;
    this.calendars = [];
    this.seemorecaption = 'See all';
    this.itemsperday = 5;
    this.weekstartday = WeekStartDay.Sunday;
    this.currentDate = new Date();
    this.intervalId = null;
  }
  onItemClicked(item) {
    this.itemClicked.emit({ item: item });
  }
  onActionClicked(item, actionId) {
    this.actionClicked.emit({ item: item, action: actionId });
  }
  render() {
    var weeks = DateUtils.getWeeksOfMonth(this.year, this.month, this.weekstartday);
    const firstDayOfMonth = new Date(this.year, this.month - 1, 1);
    const lastDayOfMonth = new Date(this.year, this.month, 0);
    return (h("div", { class: "K2BT_CalendarMonthView" }, h("div", { class: "K2BT_CalendarMonthDayNames" }, weeks[0]
      .map(d => d.getDay())
      .map(d => (h("div", { class: "K2BT_CalendarMonthViewDayName" }, DateUtils.getDayOfWeekShortStringFromDayNumber(d))))), weeks.map(w => {
      return (h("div", { class: "K2BT_CalendarMonthViewWeek" }, w.map(d => {
        var year = d.getFullYear();
        var month = d.getMonth();
        var day = d.getDate();
        var filteredAppointments = this.getFilteredAppointments(year, month, day);
        var visibleAppointments = this.itemsperday > 0 ? filteredAppointments.slice(0, this.itemsperday) : filteredAppointments;
        var dayClasses = this.getClassesForDayContainer(d, firstDayOfMonth, lastDayOfMonth);
        var dayNumberContainerClasses = this.getClassesForNumberContainer(year, month, day);
        visibleAppointments.sort((a, b) => a.dateFrom.getTime() - b.dateFrom.getTime());
        return (h("div", { class: dayClasses, onClick: () => this.onDayClicked(d) }, h("div", { class: "K2BT_CalendarMonthDayComponent" }, h("div", { class: dayNumberContainerClasses }, d.getDate()), h("div", { class: "K2BT_CalendarMonthViewDayAppointmentList" }, visibleAppointments.map(item => {
          var _a;
          var calendar = (_a = this.calendars.filter(c => c.id === item.calendarId)[0]) !== null && _a !== void 0 ? _a : this.calendars[0];
          var classList = new Array('K2BT_CalendarMonthViewDayAppointmentBadge');
          classList.push(calendar.class);
          CalendarItem.addItemClasses(classList, item, this.currentDate);
          const periodString = item.allDay ? '' : DateUtils.formatItemDate(item.dateFrom) + '-' + DateUtils.formatItemDate(item.dateTo);
          return (h("div", { class: "K2BT_CalendarMonthViewDayAppointment", onClick: e => {
              e.stopPropagation();
              this.onItemClicked(item);
            }, title: item.description + '\n' + periodString }, h("div", { class: classList.join(' ') }), h("div", { class: "K2BT_CalendarMonthViewDayAppointmentPeriod" }, periodString), h("div", { class: "K2BT_CalendarMonthViewDayAppointmentDescription" }, item.description), h("k2bt-calendar-action-menu", { actions: item.actions, onActionClicked: e => this.onActionClicked(item, e.detail) })));
        }), visibleAppointments.length < filteredAppointments.length ? h("div", { class: "K2BT_CalendarMonthViewDaySeeMore" }, this.seemorecaption) : ''))));
      })));
    })));
  }
  getClassesForNumberContainer(year, month, day) {
    var dayNumberContainerClasses = 'K2BT_CalendarMonthViewDayNumber';
    if (this.currentDate.getFullYear() === year && this.currentDate.getMonth() === month && this.currentDate.getDate() === day)
      dayNumberContainerClasses += ' K2BT_CalendarDayViewToday';
    return dayNumberContainerClasses;
  }
  getClassesForDayContainer(d, firstDayOfMonth, lastDayOfMonth) {
    var dayClasses = 'K2BT_CalendarMonthViewDay';
    if (!(d >= firstDayOfMonth && d <= lastDayOfMonth)) {
      dayClasses += ' K2BT_CalendarMonthViewDayOtherMonth';
    }
    if (d < new Date(this.currentDate.getFullYear(), this.currentDate.getMonth(), this.currentDate.getDate())) {
      dayClasses += ' K2BT_CalendarMonthViewDayPast';
    }
    return dayClasses;
  }
  getFilteredAppointments(year, month, day) {
    return this.calendars
      .map(c => c.items
      .filter(i => (i.dateFrom.getFullYear() == year && i.dateFrom.getMonth() == month && i.dateFrom.getDate() == day) ||
      (i.dateTo.getFullYear() == year && i.dateTo.getMonth() == month && i.dateTo.getDate() == day))
      .map(i => {
      i.calendarId = c.id;
      return i;
    }))
      .reduce((i1, i2) => i1.concat(i2), []);
  }
  componentDidLoad() {
    this.intervalId = setInterval(function () {
      this.currentDate = new Date();
    }.bind(this), 1000 * 60 * 10); // Every 10 minutes
  }
  disconnectedCallback() {
    clearInterval(this.intervalId);
  }
  onDayClicked(day) {
    this.dayClicked.emit({ date: day });
  }
};

const K2btCalendarPeriodView = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.newDraftCreated = createEvent(this, "newDraftCreated", 7);
    this.draftCanceled = createEvent(this, "draftCanceled", 7);
    this.itemClicked = createEvent(this, "itemClicked", 7);
    this.calendars = [];
    this.readonly = true;
    this.draftItemDescription = '(no title)';
    this.draftItem = null;
    this.dayComponents = [];
  }
  async onCreatingNewDraft(event) {
    for (let d of this.dayComponents) {
      if (event.target != d)
        await d.cancelDraft();
    }
  }
  async cancelDraft() {
    for (let d of this.dayComponents) {
      await d.cancelDraft();
    }
  }
  render() {
    var days = [];
    var d = new Date(this.dateFrom.getTime());
    while (d < this.dateTo) {
      days.push({ year: d.getFullYear(), month: d.getMonth(), day: d.getDate() });
      d.setDate(d.getDate() + 1);
    }
    this.dayComponents = [];
    var maxDayItemsInDate = Math.max(...days
      .map(day => new Date(day.year, day.month, day.day, 0, 0, 0, 0))
      .map(date => this.calendars.map(c => c.items.filter(i => i.allDay && CalendarItem.allDayItemIsInDay(i, date))).reduce((i1, i2) => i1.concat(i2), []).length));
    var first = true;
    return (h("div", { class: "K2BT_CalendarPeriod" }, days.map(d => {
      var showhours = first;
      first = false;
      return (h("div", { class: "K2BT_CalendarPeriodDayContainer" }, h("k2bt-calendar-day-view", { year: d.year, month: d.month, day: d.day, startHour: this.startHour, endHour: this.endHour, readonly: this.readonly, calendars: this.calendars, showhours: showhours, ref: c => this.dayComponents.push(c), alldayrows: maxDayItemsInDate, onCreatingNewDraft: this.onCreatingNewDraft.bind(this) })));
    })));
  }
};

export { K2btCalendarDayInMonthPicker as k2bt_calendar_day_in_month_picker, K2btCalendarMonthView as k2bt_calendar_month_view, K2btCalendarPeriodView as k2bt_calendar_period_view };
